--[[
Power Tools for FS22

Author:     w33zl / WZL Modding
Version:    1.0.0.4
Modified:   2021-11-29

Facebook:           https://www.facebook.com/w33zl

Changelog:
]]

PowerTools = Mod:init()

-- PowerTools:enableDebugMode()

FSBaseMission.registerActionEvents = Utils.appendedFunction(FSBaseMission.registerActionEvents, function()
    local triggerUp, triggerDown, triggerAlways, startActive, callbackState, disableConflictingBindings = false, true, false, true, nil, true
    local state, actionEventId, otherEvents = g_inputBinding:registerActionEvent(InputAction.POWERTOOLSMENU, PowerTools, PowerTools.showMenu, triggerUp, triggerDown, triggerAlways, startActive, callbackState, disableConflictingBindings)
    g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW)
    -- local state, actionEventId, otherEvents = g_inputBinding:registerActionEvent(InputAction.POWERTOOLS_QUICKSAVE, PowerTools, PowerTools.saveGame, triggerUp, triggerDown, triggerAlways, startActive, callbackState, disableConflictingBindings)
    -- g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW)
    -- g_inputBinding:setActionEventTextVisibility(actionEventId, false) -- INFO: change "false" to "true" to show keybinding in help window
end)

FSBaseMission.delete = Utils.appendedFunction(FSBaseMission.delete, function()
    if PowerTools ~= nil then
        if PowerTools.delete ~= nil then
            PowerTools:delete()
        end
        removeModEventListener(PowerTools)
        PowerTools = nil -- GC
    end
end)


function PowerTools:showMenu()
    local actionFillVehicle = { g_i18n:getText("fillVehicle"), self.fillVehicle }
    local actionTipOnGround = { g_i18n:getText("tipOnGround"), self.tipToGround }
    local isInVehicle = g_currentMission.controlledVehicle ~= nil
    local actions = {
        -- (isInVehicle == true and actionFillVehicle) or actionTipOnGround,
        { g_i18n:getText("superStrengthMode"), self.toggleSuperStrength },
        { g_i18n:getText("flightMode"), self.toggleFlightMode },
        { g_i18n:getText("noHudMode"), self.toggleHUDMode },
        { g_i18n:getText("changeMoneyMode"), self.addRemoveMoney },
        -- { g_i18n:getText("saveGame"), self.saveGame },
        { g_i18n:getText("exitMode"), self.exitSavegame },
    }

    --TODO: replace when TipOnGround works
    if isInVehicle == true then
        table.insert( actions, 1, actionFillVehicle )
    end


    local options = {}
    for index, value in ipairs(actions) do
        options[#options + 1] = index .. ") " .. value[1]
    end

    local dialogArguments = {
        text = g_i18n:getText("chooseAction"),
        title = g_i18n:getText("powerTools"),
        options = options,
        target = self,
        args = { },
        callback = function(target, selectedOption, a)
            if type(selectedOption) ~= "number" or selectedOption == 0 then
                return
            end

            local delegate = actions[selectedOption][2]

            delegate(self)
        end,
    }

    --TODO: hack to reset the "remembered" option (i.e. solve a bug in the game engine)
    local dialog = g_gui.guis["OptionDialog"]
    if dialog ~= nil then
        dialog.target:setOptions({""}) -- Add fake option to force a "reset"
    end

    g_gui:showOptionDialog(dialogArguments)
    

end
function PowerTools:tipToGround()

end


function PowerTools:toggleSuperStrength()
    g_currentMission.player:consoleCommandToggleSuperStrongMode()

    if g_currentMission.player.superStrengthEnabled then
        g_currentMission:addGameNotification(g_i18n:getText("superStrength") .. ": " .. g_i18n:getText("enabled"), "", "")
    else
        g_currentMission:addGameNotification(g_i18n:getText("superStrength") .. ": " .. g_i18n:getText("disabled"), "", "")
    end
end

function PowerTools:toggleHUDMode()
    g_currentMission.hud:consoleCommandToggleVisibility()
    -- g_currentMission.hud.isVisible = not (g_currentMission.hud.isVisible or false)  -- thirdPersonViewActive
end

function PowerTools:toggleFlightMode()
    g_currentMission.player:consoleCommandToggleFlightMode()

    if g_flightModeEnabled then
        g_currentMission:addGameNotification(g_i18n:getText("flightMode") .. ": " .. g_i18n:getText("enabled"), g_i18n:getText("flightModeUsage"), "", 2500)
        g_currentMission.player.debugFlightModeWalkingSpeed = 0.032
        g_currentMission.player.debugFlightModeRunningFactor = 4

        PowerTools.maxWalkingSpeed = PowerTools.maxWalkingSpeed or g_currentMission.player.motionInformation.maxWalkingSpeed
        g_currentMission.player.motionInformation.maxWalkingSpeed = 12
    
        g_currentMission.player:onInputDebugFlyToggle()
    else
        g_currentMission:addGameNotification(g_i18n:getText("flightMode") .. ": " .. g_i18n:getText("disabled"), "", "", 1500)
        g_currentMission.player.motionInformation.maxWalkingSpeed = PowerTools.maxWalkingSpeed or g_currentMission.player.motionInformation.maxWalkingSpeed
        PowerTools.maxWalkingSpeed = nil
    end
end

function PowerTools:addRemoveMoney()
    local dialogArguments = {
        text = g_i18n:getText("changeMoneyMode"):upper() .. ":\n\n" .. g_i18n:getText("changeMoneyUsage"):gsub("\\\\n", "\n"),
        target = self, 
        defaultText = "", 
        maxCharacters = 10, 
        args = {}, 
        disableFilter = true,
        confirmText = g_i18n:getText("changeMoneyMode"),
        backText = g_i18n:getText("buttonCancel"),
    }

    dialogArguments.callback = function(target, value, arguments)
        if value == nil or value == "" then return end

        local function addMoney(amount, isAbsolute)
            local moneyChange = amount

            if isAbsolute then
                local farm = g_farmManager:getFarmById(g_currentMission.player.farmId)
                local refMoney = farm.money
                moneyChange = amount - refMoney
            end

            g_currentMission:addMoney(moneyChange, g_currentMission.player.farmId, MoneyType.OTHER)
            g_currentMission:addMoneyChange(moneyChange, g_currentMission.player.farmId, MoneyType.OTHER, true)

        end

        local numericValue, isExact = 0, false

        if value:find("=") == 1 then
            value = value:sub(2, -1)
            isExact = true
        end

        numericValue = tonumber(value)

        if numericValue ~= nil and type(numericValue) == "number" then
            addMoney(numericValue, isExact)
        else
            g_currentMission:showBlinkingWarning(g_i18n:getText("errorNotNumeric") .. tostring(value))
        end
    end

    g_gui:showTextInputDialog(dialogArguments)

end

function PowerTools:loadMap()
    addConsoleCommand("ee", "Exit to the start menu", "consoleCommandExitSavegame", self)
    addConsoleCommand("qq", "Quit the game", "commandQuitGame", self)
end

function PowerTools:delete()
    Logging.info("Unloading mod Power Tools")
    removeConsoleCommand("ee")
    removeConsoleCommand("qq")
    
end

local function doExitRestartGame()
    PowerTools:printInfo("Exiting to main menu/restarting current game")
    PowerTools:delete()
    removeModEventListener(PowerTools)
    -- g_gui:showGui("InGameMenu")
    g_currentMission.inGameMenu:leaveCurrentGame()
    g_gui:showGui("")
end

function PowerTools:exitSavegame()
    g_gui:showYesNoDialog({
        title = g_i18n:getText("confirmExit"),
        text = g_i18n:getText("exitRestartWarning"),
        callback =  function(self, yes)
            PowerTools:printDebugVar("doExit?", yes)
            if yes == true then
                doExitRestartGame() 
            end
        end,
        target = self
    })
end

function PowerTools:consoleCommandExitSavegame()
    doExitRestartGame()
end

function PowerTools:commandQuitGame()
    doExit()
end

function PowerTools:showOptionDialog(text, title, options, callback)

    --TODO: hack to reset the "remembered" option (i.e. solve a bug in the game engine)
    local dialog = g_gui.guis["OptionDialog"]
    if dialog ~= nil then
        dialog.target:setOptions({""}) -- Add fake option to force a "reset"
    end

    g_gui:showOptionDialog({
        text = text,
        title = title,
        defaultText = "",
        options = options,
        defaultOption = 1,
        target = self,
        args = { },
        callback = callback,
    })
end

function PowerTools:fillFillUnit(selectedFillUnit)
    if selectedFillUnit == nil then
        g_currentMission:showBlinkingWarning("No/invalid fillunit index")
        return
    end

    local options = {}
    local optionToFilltypeIndex = {}
    options[#options + 1] = g_i18n:getText("filltypeNone"):upper()
    optionToFilltypeIndex[#optionToFilltypeIndex + 1] = 0

    for fillTypeIndex, _ in pairs(selectedFillUnit.supportedFillTypes) do
        local fillTypeName = g_fillTypeManager:getFillTypeNameByIndex(fillTypeIndex)

        if self.debugMode then fillTypeName = fillTypeName .. " [" .. tostring(fillTypeIndex) .. "]" end

        options[#options + 1] = fillTypeName
        optionToFilltypeIndex[#optionToFilltypeIndex + 1] = fillTypeIndex
    end

    if #options < 1 then
        g_currentMission:showBlinkingWarning(g_i18n:getText("errorNoValidFilltypes"))
        return
    end


    local dialogArguments = {
        text = g_i18n:getText("selectFillType"),
        title = g_i18n:getText("fillVehicle"),
        -- okTexr = "Spara",
        options = options,
        target = self,
        args = { },
        callback = function(target, selectedOption, a)

            PowerTools:printDebugVar("selectedOption", selectedOption)
            if selectedOption > 0 then

                local selectedFillUnitIndex = selectedFillUnit.fillUnitIndex
                local selectedFillTypeIndex = optionToFilltypeIndex[selectedOption]
                local amount = selectedFillUnit.capacity or 1000

                PowerTools:printDebugVar("selectedFillUnit.capacity", selectedFillUnit.capacity)
                PowerTools:printDebugVar("selectedFillTypeIndex", selectedFillTypeIndex)

                -- Clean first...
                local currentFillTypeIndex = selectedFillUnit.fillType or selectedFillUnit.supportedFillTypes[1]
                FSBaseMission.consoleCommandFillUnitAdd(g_currentMission, selectedFillUnitIndex, g_fillTypeManager:getFillTypeNameByIndex(currentFillTypeIndex), 0)

                if selectedFillTypeIndex > 0 then
                    local selectedFillTypeName = g_fillTypeManager:getFillTypeNameByIndex(selectedFillTypeIndex)
                    PowerTools:printDebugVar("selectedFillTypeName", selectedFillTypeName)
                    PowerTools:printDebugVar("amount", amount)

                    FSBaseMission.consoleCommandFillUnitAdd(g_currentMission, selectedFillUnitIndex, selectedFillTypeName, amount)
                end

            end
        end,
    }

    g_gui:showOptionDialog(dialogArguments)

end

function PowerTools:fillVehicle()

    local controlledVehicle = g_currentMission.controlledVehicle
    local selectedVehicle = controlledVehicle ~= nil and controlledVehicle:getSelectedVehicle()
    local currentVehicle = selectedVehicle or controlledVehicle
    local spec_fillUnit = currentVehicle ~= nil and currentVehicle.spec_fillUnit

    -- Pre scan
    local validFillUnits = {}
    local function checkFillUnitsForVehicle(vehicle)
        local spec_fillUnit = vehicle ~= nil and vehicle.spec_fillUnit
        local perVehicleIndex = 0

        if spec_fillUnit == nil then
            return
        end

        local function addFillUnitOption(index, fillUnit)
            perVehicleIndex = perVehicleIndex + 1
            local fillUnitOption = {
                index = index,
                fillType = fillUnit.fillType,
                fillLevel = fillUnit.fillLevel,
                capacity = fillUnit.capacity,
                unitText = fillUnit.unitText,
                vehiclePrefix = vehicle.typeDesc:upper() .. "[" .. tostring(perVehicleIndex) .. "]"
            }
            
            validFillUnits[#validFillUnits + 1] = fillUnitOption

        end
        
        
        for index, fillUnit in ipairs(spec_fillUnit.fillUnits) do
            if fillUnit.supportedFillTypes ~= nil and fillUnit.showOnInfoHud then
                if #fillUnit.supportedFillTypes > 0 then
                    addFillUnitOption(index, fillUnit)
                else-- Second chance
                    for k, v in pairs(fillUnit.supportedFillTypes) do
                        if type(k) == "number" and type(v) == "boolean" and v == true then
                            addFillUnitOption(index, fillUnit)
                            break
                        end
                    end
                end
            end
            
        end
    end
    
    checkFillUnitsForVehicle(currentVehicle)
    -- if currentVehicle ~= controlledVehicle then
    --     checkFillUnitsForVehicle(controlledVehicle)
    -- end


    local selectedFillUnitIndex
    if #validFillUnits == 0 then
        g_currentMission:showBlinkingWarning(g_i18n:getText("errorNoValidFillUnits"))
        return
    elseif #validFillUnits == 1 then
        selectedFillUnitIndex = 1
        self:fillFillUnit(spec_fillUnit.fillUnits[1])
    else
        local options = {}
        for index, value in ipairs(validFillUnits) do
            local name = value.fillType == 0 and "-" or g_fillTypeManager:getFillTypeNameByIndex(value.fillType)
            -- DebugUtil.printTableRecursively(value, "value:: ", 0, 2)
            options[#options + 1] = string.format( "%s: %s (%d/%d)", value.vehiclePrefix, name, value.fillLevel, value.capacity)
        end

        self:showOptionDialog(
            g_i18n:getText("chooseFillUnitDetailed"),
            g_i18n:getText("chooseFillUnit"),
            options,
            function(target, selectedOption)
                if selectedOption > 0 then
                    self:fillFillUnit(spec_fillUnit.fillUnits[selectedOption])
                end
            end
        )
    end
end

function PowerTools:saveGame()
    
end